"""
This code is part of GOsa (http://www.gosa-project.org)
Copyright (C) 2009, 2010 GONICUS GmbH

ID: $$Id: command.py 281 2010-07-02 07:43:23Z cajus $$

This is the zeroconf provider module.

See LICENSE for more information about the licensing.
"""

# Global command types
NORMAL = 1
FIRSTRESULT = 2
CUMULATIVE = 4


def Command(**kwds):
    """
    This is the Command decorator. It adds properties based on its
    parameters to the function attributes.

    Example: @Command(needsQueue= False, type= NORMAL)

    @type needsQueue: bool
    @param needsQueue: indicates if the decorated function needs
                       a queue parameter

    @type type: int
    @param type: describes the function type

    Function types can be:

    * NORMAL (default)
      The decorated function will be called as if it is local. Which
      node will answer this request is not important.

    * FIRSTRESULT
      Some functionality may be distributed on several nodes with
      several information. FIRSTRESULT iterates thru all nodes which
      provide the decorated function and return on first success.

    * CUMULATIVE
      Some functionality may be distributed on several nodes with
      several information. CUMULATIVE iterates thru all nodes which
      provide the decorated function and returns the combined result.
    """
    def decorate(f):
        setattr(f, 'isCommand', True)
        for k in kwds:
            setattr(f, k, kwds[k])
        return f
    return decorate


class CommandInvalid(Exception):
    """ Exception which is raised when the command is not valid. """
    pass


class CommandNotAuthorized(Exception):
    """ Exception which is raised when the call was not authorized. """
    pass
