# -*- coding: utf-8 -*-
"""
 This code is part of GOsa (http://www.gosa-project.org)
 Copyright (c) 2007 Jan-Klaas Kollhof
 Copyright (C) 2010 GONICUS GmbH

 ID: $$Id: jsonrpc_proxy.py 608 2010-08-16 08:12:35Z cajus $$

 This file is part of jsonrpc. We have modified it to be able to
 use session based authentication.

 jsonrpc is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 This software is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with this software; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
import os
import os.path
import urllib2
import cookielib
from jsonrpc.json import dumps, loads


class JSONRPCException(Exception):

    def __init__(self, rpcError):
        Exception.__init__(self)
        self.error = rpcError


class JSONServiceProxy(object):
    """
    The JSONServiceProxy provides a simple way to use GOsa RPC
    services from various clients. Using the proxy object, you
    can directly call methods without the need to know where
    it actually gets executed.

    Example:

    proxy = JSONServiceProxy('https://localhost')
    print(proxy.login("admin", "secret"))
    print(proxy.getMethods())
    print(proxy.logout())

    This will list the available methods.
    """

    def __init__(self, serviceURL=None, serviceName=None, opener=None):
        """
        Instantiate the proxy object using the initializing parameters.

        @type serviceURL: string
        @param serviceURL: URL used to connect to the GOsa service
           http(s)://host:port
        """
        self.__serviceURL = serviceURL
        self.__serviceName = serviceName

        if not opener:
            http_handler = urllib2.HTTPHandler()
            https_handler = urllib2.HTTPSHandler()
            cookie_handler = urllib2.HTTPCookieProcessor(cookielib.CookieJar())
            opener = urllib2.build_opener(http_handler, https_handler, cookie_handler)

        self.__opener = opener

    def __getattr__(self, name):
        if self.__serviceName != None:
            name = "%s.%s" % (self.__serviceName, name)

        return JSONServiceProxy(self.__serviceURL, name, self.__opener)

    def __call__(self, *args):
        postdata = dumps({"method": self.__serviceName, 'params': args, 'id': 'jsonrpc'})
        respdata = self.__opener.open(self.__serviceURL, postdata).read()
        resp = loads(respdata)
        if resp['error'] != None:
            raise JSONRPCException(resp['error'])
        else:
            return resp['result']
